# -*- coding: utf-8 -*-
#
# Client for OX' SOAP API
#
# Copyright 2018 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

#
# Client for OX' SOAP API
# API docs: http://oxpedia.org/wiki/index.php?title=Open-Xchange_Provisioning_using_SOAP
#

#
# INSTALL
#
# Debain Stretch: python-zeep 0.23.0-1
# Debain Buster:  python-zeep 2.4.0-1   # <-- we want this
#
# UCS 4.2:
#
# univention-install virtualenv libxslt-dev libxml2-dev libz-dev
# virtualenv --system-site-packages venv_zeep
# . venv_zeep/bin/activate
# pip install zeep
#

from __future__ import absolute_import

import pprint
import random

from zeep import Client as ZeepClient
from zeep.cache import InMemoryCache
from zeep.transports import Transport

from univention.ox.http.client import Client as OXHttpApi
from univention.ox.listener_tools import get_credentials_for_context, get_master_credentials

try:
	from typing import Any, Optional, Dict, List, Set, Type
	import types.TracebackType
except ImportError:
	pass


WS_BASE_URL = 'http://127.0.0.1/webservices'
WS_CONTEXT_URL = '{}/OXContextService?wsdl'.format(WS_BASE_URL)
WS_GROUP_URL = '{}/OXGroupService?wsdl'.format(WS_BASE_URL)
WS_USER_URL = '{}/OXUserService?wsdl'.format(WS_BASE_URL)

CONTEXT = 10

transport = Transport(cache=InMemoryCache())

client = ZeepClient(WS_CONTEXT_URL, transport=transport)
credentials_type = client.wsdl.types.get_type('{http://dataobjects.rmi.admin.openexchange.com/xsd}Credentials')
context_type = client.wsdl.types.get_type('{http://dataobjects.soap.admin.openexchange.com/xsd}Context')
user_type = client.wsdl.types.get_type('{http://dataobjects.soap.admin.openexchange.com/xsd}User')

context = context_type(id=CONTEXT)
context_creds = credentials_type(*get_credentials_for_context(CONTEXT))
master_creds = credentials_type(*get_master_credentials())

print('*** All contexts:')
pp = pprint.PrettyPrinter(indent=2)
pp.pprint(client.service.listAll(auth=master_creds))

print('*** All groups in context {}:'.format(CONTEXT))
client = ZeepClient(WS_GROUP_URL, transport=transport)
pp.pprint(client.service.listAll(ctx=context, auth=context_creds))

print('*** All groups in context {}:'.format(CONTEXT))
client = ZeepClient(WS_GROUP_URL, transport=transport)
pp.pprint(client.service.listAll(ctx=context, auth=context_creds))

print('*** All users in context {}:'.format(CONTEXT))
client = ZeepClient(WS_USER_URL, transport=transport)
users = client.service.listAll(ctx=context, auth=context_creds)
print('*** IDs: {}'.format([int(user.id) for user in users]))
# for user in users:
#    for k, v in user.__values__.items():
# 	   if v is not None:
# 		   print('{!r} : {!r}'.format(k, v))
#    print('---')
user = user_type(id=random.choice([int(user.id) for user in users[1:]]))  # admin ist 1st user, don't delete admin
print('*** User information in listAll() is pretty low, getData() for user {}:'.format(user.id))
pp.pprint(client.service.getData(ctx=context, user=user, auth=context_creds))

kwargs = dict(id=user.id, city_home='Berlin', city_business='Bremen', business_category='bus cat')
user_mod = user_type(**kwargs)
print('*** Modifying user: {!r}'.format(kwargs))
client.service.change(ctx=context, usrdata=user_mod, auth=context_creds)

print('*** Checking modification with HTTP-API...')
client = OXHttpApi(*get_credentials_for_context(str(CONTEXT)), ssl=False)
ox_http_user = client.get_user(user.id)
pp.pprint(ox_http_user)
if all((
		ox_http_user['city_home'] == 'Berlin',
		ox_http_user['city_business'] == 'Bremen',
		ox_http_user['busines_categorie'] == 'bus cat'  # ROFL: busines_categorie
	)):
	print('*** OK')
else:
	print('*** FAIL')
